@@ -7,8 +7,8 @@
  */
 
 function Torrent( transferListParent, fileListParent, controller, data) {
-    this.initialize( transferListParent, fileListParent, controller, data);
-} 
+	this.initialize( transferListParent, fileListParent, controller, data);
+}
 
 // Constants
 Torrent._StatusWaitingToCheck  = 1;
@@ -18,6 +18,23 @@ Torrent._StatusSeeding         = 8;
 Torrent._StatusPaused          = 16;
 Torrent._InfiniteTimeRemaining = 215784000; // 999 Hours - may as well be infinite
 
+Torrent._RatioUseGlobal        = 0;
+Torrent._RatioUseLocal         = 1;
+Torrent._RatioUnlimited        = 2;
+
+Torrent._ErrNone               = 0;
+Torrent._ErrTrackerWarning     = 1;
+Torrent._ErrTrackerError       = 2;
+Torrent._ErrLocalError         = 3;
+
+Torrent._StaticFields = [ 'addedDate', 'announceURL', 'comment', 'creator',
+    'dateCreated', 'hashString', 'id', 'isPrivate', 'name', 'totalSize' ]
+Torrent._DynamicFields = [ 'downloadedEver', 'error', 'errorString', 'eta',
+    'haveUnchecked', 'haveValid', 'leechers', 'leftUntilDone', 'peersConnected',
+    'peersGettingFromUs', 'peersSendingToUs', 'rateDownload', 'rateUpload',
+    'recheckProgress', 'seeders', 'sizeWhenDone', 'status', 'swarmSpeed',
+    'uploadedEver', 'uploadRatio', 'seedRatioLimit', 'seedRatioMode', 'downloadDir' ]
+
 Torrent.prototype =
 {
 	/*
@@ -85,7 +102,7 @@ Torrent.prototype =
 		e.appendChild( image );
 		top_e.appendChild( e );
 		element._pause_resume_button_image = image;
-		if (!iPhone) $(e).bind('click', {element: element}, this.clickPauseResumeButton);
+		if (!iPhone) $(e).bind('click', function(e) { element._torrent.clickPauseResumeButton(e); });
 		
 		// Create the 'peer details' <div>
 		e = document.createElement( 'div' );
@@ -94,7 +111,7 @@ Torrent.prototype =
 		element._peer_details_container = e;
 		
 		// Set the torrent click observer
-		element.bind('click', {element: element}, this.clickTorrent);
+		element.bind('click', function(e){ element._torrent.clickTorrent(e) });
 		
 		// Safari hack - first torrent needs to be moved down for some reason. Seems to be ok when
 		// using <li>'s in straight html, but adding through the DOM gets a bit odd.
@@ -134,9 +151,9 @@ Torrent.prototype =
 	},
 	
 	/*--------------------------------------------
-	 * 
+	 *
 	 *  S E T T E R S   /   G E T T E R S
-	 * 
+	 *
 	 *--------------------------------------------*/
 	
 	/* Return the DOM element for this torrent (a <LI> element) */
@@ -158,7 +175,6 @@ Torrent.prototype =
 	dateAdded: function() { return this._date; },
 	downloadSpeed: function() { return this._download_speed; },
 	downloadTotal: function() { return this._download_total; },
-	errorMessage: function() { return this._error_message; },
 	hash: function() { return this._hashString; },
 	id: function() { return this._id; },
 	isActive: function() { return this.state() != Torrent._StatusPaused; },
@@ -201,11 +217,18 @@ Torrent.prototype =
 		this.fileList().show();
 	},
 	hideFileList: function() { this.fileList().hide(); },
+	seedRatioLimit: function(){
+		switch( this._seed_ratio_mode ) {
+			case Torrent._RatioUseGlobal: return this._controller.seedRatioLimit();
+			case Torrent._RatioUseLocal:  return this._seed_ratio_limit;
+			default:                      return -1;
+		}
+	},
 	
 	/*--------------------------------------------
-	 * 
+	 *
 	 *  E V E N T   F U N C T I O N S
-	 * 
+	 *
 	 *--------------------------------------------*/
 	
 	/*
@@ -216,7 +239,7 @@ Torrent.prototype =
 		// Prevents click carrying to parent element
 		// which deselects all on click
 		event.stopPropagation();
-		var torrent = event.data.element._torrent;
+		var torrent = this;
 		
 		// 'Apple' button emulation on PC :
 		// Need settable meta-key and ctrl-key variables for mac emulation
@@ -229,6 +252,8 @@ Torrent.prototype =
 		
 		// Shift-Click - Highlight a range between this torrent and the last-clicked torrent
 		if (iPhone) {
+			if ( torrent.isSelected() )
+				torrent._controller.showInspector();
 			torrent._controller.setSelectedTorrent( torrent, true );
 		
 		} else if (event.shiftKey) {
@@ -265,7 +290,7 @@ Torrent.prototype =
 		event.stopPropagation();
 
 		// either stop or start the torrent
-		var torrent = event.data.element._torrent;
+		var torrent = this;
 		if( torrent.isActive( ) )
 			torrent._controller.stopTorrent( torrent );
 		else
@@ -273,9 +298,9 @@ Torrent.prototype =
 	},
 
 	/*--------------------------------------------
-	 * 
+	 *
 	 *  I N T E R F A C E   F U N C T I O N S
-	 * 
+	 *
 	 *--------------------------------------------*/
 	
 	refresh: function(data) {
@@ -292,18 +317,24 @@ Torrent.prototype =
 		this._leftUntilDone         = data.leftUntilDone;
 		this._download_total        = data.downloadedEver;
 		this._upload_total          = data.uploadedEver;
+		this._upload_ratio          = data.uploadRatio;
+		this._seed_ratio_limit      = data.seedRatioLimit;
+		this._seed_ratio_mode       = data.seedRatioMode;
 		this._download_speed        = data.rateDownload;
 		this._upload_speed          = data.rateUpload;
 		this._peers_connected       = data.peersConnected;
 		this._peers_getting_from_us = data.peersGettingFromUs;
 		this._peers_sending_to_us   = data.peersSendingToUs;
+		this._sizeWhenDone          = data.sizeWhenDone;
+		this._recheckProgress       = data.recheckProgress;
 		this._error                 = data.error;
-		this._error_message         = data.errorString;
+		this._error_string          = data.errorString;
 		this._eta                   = data.eta;
 		this._swarm_speed           = data.swarmSpeed;
 		this._total_leechers        = Math.max( 0, data.leechers );
 		this._total_seeders         = Math.max( 0, data.seeders );
 		this._state                 = data.status;
+		this._download_dir          = data.downloadDir;
 
 		if (data.fileStats)
 			this.refreshFileModel( data );
@@ -321,18 +352,77 @@ Torrent.prototype =
 		}
 	},
 
+	getErrorMessage: function()
+	{
+		if( this._error  == Torrent._ErrTrackerWarning )
+			return 'Tracker returned a warning: ' + this._error_string;
+		if( this._error  == Torrent._ErrTrackerError )
+			return 'Tracker returned an error: ' + this._error_string;
+		if( this._error  == Torrent._ErrLocalError )
+			return 'Error: ' + this._error_string;
+		return null;
+	},
+
+	getPeerDetails: function()
+	{
+		var c;
+
+		if(( c = this.getErrorMessage( )))
+			return c;
+
+		var st = this.state( );
+		switch( st )
+		{
+			case Torrent._StatusPaused:
+			case Torrent._StatusWaitingToCheck:
+				c = this.stateStr( );
+				break;
+
+			case Torrent._StatusDownloading:
+				// 'Downloading from 36 of 40 peers - DL: 60.2 KB/s UL: 4.3 KB/s'
+				c = 'Downloading from ';
+				c += this.peersSendingToUs();
+				c += ' of ';
+				c += this._peers_connected;
+				c += ' peers - DL: ';
+				c += Math.formatBytes(this._download_speed);
+				c += '/s UL: ';
+				c += Math.formatBytes(this._upload_speed);
+				c += '/s';
+				break;
+
+			case Torrent._StatusSeeding:
+				// 'Seeding to 13 of 22 peers - UL: 36.2 KB/s'
+				c = 'Seeding to ';
+				c += this.peersGettingFromUs();
+				c += ' of ';
+				c += this._peers_connected;
+				c += ' peers - UL: ';
+				c += Math.formatBytes(this._upload_speed);
+				c += '/s';
+				break;
+
+			case Torrent._StatusChecking:
+				// 'Verifying local data (40% tested)'
+				c = 'Verifying local data (';
+				c += Math.roundWithPrecision( 100.0 * this._recheckProgress, 0 );
+				c += '% tested)';
+				break;
+		}
+		return c;
+	},
+
 	refreshHTML: function() {
 		var c;
 		var e;
 		var progress_details;
-		var peer_details;
 		var root = this._element;
 		var MaxBarWidth = 100; // reduce this to make the progress bar shorter (%)
 		
 		setInnerHTML( root._name_container, this._name );
 		
 		// Add the progress bar
-                var notDone = this._leftUntilDone > 0;
+		var notDone = this._leftUntilDone > 0;
 
 		// Fix for situation
 		// when a verifying/downloading torrent gets state seeding
@@ -345,7 +435,7 @@ Torrent.prototype =
 			
 			if( this.isActive( ) )
 			{
-				eta = '-';
+				eta = ' - ';
 				if (this._eta < 0 || this._eta >= Torrent._InfiniteTimeRemaining )
 					eta += 'remaining time unknown';
 				else
@@ -378,65 +468,52 @@ Torrent.prototype =
 			e = root._progress_incomplete_container;
 			if( e.className.indexOf( 'incomplete' ) === -1 )
 				e.className = 'torrent_progress_bar in_progress';
+			// Clear the 'seeding' tag 
+			if( e.className.indexOf( 'seeding' ) != -1 ) 
+				e.className = 'torrent_progress_bar incomplete';
 			e.style.width =  (MaxBarWidth - css_completed_width) + '%';
 			e.style.display = 'block';
-			
-			// Create the 'peer details' label
-			// Eg: 'Downloading from 36 of 40 peers - DL: 60.2 KB/s UL: 4.3 KB/s'
-			if( !this.isDownloading( ) )
-				c = this.stateStr( );
-			else {
-				c = 'Downloading from ';
-				c += this.peersSendingToUs();
-				c += ' of ';
-				c += this._peers_connected;
-				c += ' peers - DL: ';
-				c += Math.formatBytes(this._download_speed);
-				c += '/s UL: ';
-				c += Math.formatBytes(this._upload_speed);
-				c += '/s';
-			}
-			peer_details = c;
-			
 		}
 		else
 		{
-			// Update the 'in progress' bar
-			e = root._progress_complete_container;
-			c = 'torrent_progress_bar';
-			c += (this.isActive()) ? ' complete' : ' complete_stopped';
-			e.className = c;
-			
 			// Create the 'progress details' label
 			// Eg: '698.05 MB, uploaded 8.59 GB (Ratio: 12.3)'
 			c = Math.formatBytes( this._size );
 			c += ', uploaded ';
 			c += Math.formatBytes( this._upload_total );
 			c += ' (Ratio ';
-			c += Math.ratio( this._upload_total, this._download_total );
+			c += Math.round(this._upload_ratio*100)/100;
 			c += ')';
 			progress_details = c;
-			
-			// Hide the 'incomplete' bar
-			root._progress_incomplete_container.style.display = 'none';
-			
-			// Set progress to maximum
-			root._progress_complete_container.style.width =  MaxBarWidth + '%';
-			
-			// Create the 'peer details' label
-			// Eg: 'Seeding to 13 of 22 peers - UL: 36.2 KB/s'
-			if( !this.isSeeding( ) )
-				c = this.stateStr( );
-			else {
-				c = 'Seeding to ';
-				c += this.peersGettingFromUs();
-				c += ' of ';
-				c += this._peers_connected;
-				c += ' peers - UL: ';
-				c += Math.formatBytes(this._upload_speed);
-				c += '/s';
+
+			var status = this.isActive() ? 'complete' : 'complete_stopped';
+
+			if(this.isActive() && this.seedRatioLimit() > 0){
+				status = 'complete seeding'
+				var seedRatioRatio = this._upload_ratio / this.seedRatioLimit();
+				var seedRatioPercent = Math.round( seedRatioRatio * 100 * MaxBarWidth ) / 100;
+
+				// Set progress to percent seeded
+				root._progress_complete_container.style.width =	seedRatioPercent + '%';
+
+				// Update the 'incomplete' bar
+				root._progress_incomplete_container.className = 'torrent_progress_bar incomplete seeding'
+				root._progress_incomplete_container.style.display = 'block';
+				root._progress_incomplete_container.style.width = MaxBarWidth - seedRatioPercent + '%';
 			}
-			peer_details = c;
+			else
+			{
+				// Hide the 'incomplete' bar
+				root._progress_incomplete_container.className = 'torrent_progress_bar incomplete'
+				root._progress_incomplete_container.style.display = 'none';
+
+				// Set progress to maximum
+				root._progress_complete_container.style.width =	MaxBarWidth + '%';
+			}
+
+			// Update the 'in progress' bar
+			e = root._progress_complete_container;
+			e.className = 'torrent_progress_bar ' + status;
 		}
 		
 		// Update the progress details
@@ -452,13 +529,7 @@ Torrent.prototype =
 			e.className = "torrent_pause";
 		}
 		
-		if( this._error_message &&
-		    this._error_message !== '' &&
-		    this._error_message !== 'other' ) {
-			peer_details = this._error_message;
-		}
-		
-		setInnerHTML( root._peer_details_container, peer_details );
+		setInnerHTML( root._peer_details_container, this.getPeerDetails( ) );
 	
 		this.refreshFileView( );	
 	},
@@ -737,7 +808,8 @@ TorrentFile.prototype = {
 	setWanted: function(wanted) {
 		this._dirty = true;
 		this._wanted = wanted;
-		this.element().toggleClass( 'skip', !wanted );
+		if(!iPhone)
+		  this.element().toggleClass( 'skip', !wanted );
 		var command = wanted ? 'files-wanted' : 'files-unwanted';
 		this._torrent._controller.changeFileCommand(command, this._torrent, this);
 	},
@@ -797,9 +869,18 @@ TorrentFile.prototype = {
 		}
 
 		var prio;
-		if( x < 12 ) prio = -1;
-		else if( x < 23 ) prio = 0;
-		else prio = 1;
+		if(iPhone)
+		{
+			if( x < 8 ) prio = -1;
+			else if( x < 27 ) prio = 0;
+			else prio = 1;
+		}
+		else
+		{
+			if( x < 12 ) prio = -1;
+			else if( x < 23 ) prio = 0;
+			else prio = 1;
+		}
 		this.setPriority( prio );
 	}
 };
